use serde::{ Serialize, Deserialize };

use fltk::*;
use fltk::prelude::*;
use fltk::enums::*;

use std::cell::RefCell;
use std::rc::Rc;
use std::collections::HashMap;

type Id = usize;
#[derive(Serialize, Deserialize, Debug)]
struct Node {
    pub x: f32,
    pub y: f32,

    pub name: String,
    pub edges: Vec<Id>,
}
#[derive(Serialize, Deserialize, Debug)]
struct Board {
    nodes: HashMap<Id, Node>,
    image: Option<String>,
}
impl Board {
    pub fn new() -> Self {
	let nodes = HashMap::new();
	let image = None;
	Board { nodes, image }
    }
}

fn menu_cb(m: &mut impl MenuExt) {
    todo!();
}

fn main() {
    let app = app::App::default();
    let mut win = window::Window::default()
	.with_size(400, 300);
    let mut flex = group::Flex::default()
	.size_of_parent();
    flex.set_type(group::FlexType::Column);

    // Board
    let board = Rc::new(RefCell::new(Board::new()));
    let menu = Rc::clone(&board);
    
    // Menu
    let mut menubar = menu::MenuBar::default();
    menubar.add("File/New"			, Shortcut::None, menu::MenuFlag::Normal, menu_cb);
    menubar.add("File/Open..."		, Shortcut::None, menu::MenuFlag::Normal, menu_cb);
    menubar.add("File/Open Image..."	, Shortcut::None, menu::MenuFlag::Normal, menu_cb);
    menubar.add("File/Save As ..."		, Shortcut::None, menu::MenuFlag::Normal, move |_| { println!("{}", serde_json::to_string(menu.as_ref()).unwrap()); });
    menubar.add("Edit/Edit Nodes"		, Shortcut::None, menu::MenuFlag::Normal, menu_cb);
    menubar.add("Edit/Edit Edges"		, Shortcut::None, menu::MenuFlag::Normal, menu_cb);
    flex.set_size(&mut menubar, 40);

    // Canvas
    let image = Rc::new(RefCell::new(Option::<image::SharedImage>::None));
    let bg_image = Rc::clone(&image);
    let mut frame = frame::Frame::default();
    frame.draw(move |f| {
	let mut image = bg_image.borrow_mut();
	if let Some(image) = image.as_mut() {
	    image.scale(f.w(), f.h(), true, true);
	    image.draw(f.x(), f.y(), f.w(), f.h());
	}
    });
    
    flex.end();

    win.end();
    win.make_resizable(true);
    win.show();

    app.run().unwrap();
}
